"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useCreateResource = exports.useResource = void 0;
const react_1 = require("react");
const types_1 = require("../types");
const resource_1 = require("../resource");
const areHookInputsEqual_1 = require("../utilities/areHookInputsEqual");
/**
 * This function allow to use resource without React.Suspense.
 *
 * @example
 *   const { data = [], isLoading, error } = useResource(resource);
 */
function useResource(resource, { loadingStartDelay } = {}) {
    const [, forceUpdate] = react_1.useState({});
    resource.preload();
    const data = resource.value;
    const error = resource.error;
    react_1.useLayoutEffect(() => {
        let destructed = false;
        const update = () => !destructed && forceUpdate({});
        _listenerToResource(resource, { data, error }, update);
        return () => {
            destructed = true;
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [resource]);
    const isLoadingRef = react_1.useRef(false);
    react_1.useEffect(() => {
        if (!resource.isLoading || !loadingStartDelay) {
            return;
        }
        let timeout = setTimeout(() => {
            isLoadingRef.current = true;
            forceUpdate({});
            timeout = null;
        }, loadingStartDelay);
        return () => {
            if (timeout) {
                clearTimeout(timeout);
                timeout = null;
            }
            else {
                isLoadingRef.current = false;
            }
        };
    }, [resource.isLoading, loadingStartDelay]);
    return {
        data,
        isLoading: loadingStartDelay && resource.isLoading
            ? isLoadingRef.current
            : resource.isLoading,
        error,
    };
}
exports.useResource = useResource;
function _listenerToResource(resource, resolved, callback) {
    var _a;
    if (resource.status === types_1.RESOURCE_RESOLVED &&
        resolved.data !== resource.value) {
        callback();
    }
    else if (resource.status === types_1.RESOURCE_PENDING) {
        (_a = resource.promise) === null || _a === void 0 ? void 0 : _a.then(callback);
    }
}
const emptyResource = resource_1.createResource(() => undefined);
/**
 * A hook for creating resource without preloading
 *
 * @example
 *   const { resource, refetch } = useCreateResource(fetchApi, accountID, amount);
 *   resource.preload(); // if you need to preload data call this
 *
 * @param fetchFunc Promise returned function
 * @param arg This is fetchFunc arguments, if arguments changed function give
 *     another resource
 */
const useCreateResource = (fetchFunc, 
/** Default is [] */
deps = [], { isPreloadAfterCallRefetch = true, startFetchAtFirstRender = true, keepDataAliveWhenFetching = true, } = {}) => {
    const [, forceUpdate] = react_1.useState({});
    const preDepsRef = react_1.useRef(deps);
    const funcRef = react_1.useRef(fetchFunc);
    const resourceRef = react_1.useRef(emptyResource);
    if (funcRef.current !== fetchFunc) {
        funcRef.current = fetchFunc;
    }
    const isArgChanged = areHookInputsEqual_1.areHookInputsEqual(deps, preDepsRef.current);
    if (!isArgChanged) {
        preDepsRef.current = deps;
    }
    if (!isArgChanged ||
        (resourceRef.current === emptyResource && startFetchAtFirstRender)) {
        resourceRef.current = resource_1.createResource(fetchFunc, keepDataAliveWhenFetching ? resourceRef.current.value : undefined);
    }
    const refetch = react_1.useCallback((...args) => {
        resourceRef.current = resource_1.createResource((() => funcRef.current(...args)), keepDataAliveWhenFetching ? resourceRef.current.value : undefined);
        isPreloadAfterCallRefetch && resourceRef.current.preload();
        forceUpdate({});
    }, [isPreloadAfterCallRefetch, keepDataAliveWhenFetching]);
    return { resource: resourceRef.current, refetch };
};
exports.useCreateResource = useCreateResource;
//# sourceMappingURL=resource.js.map